% DISK_ORIGIN
% This script measures the error made by discretizing a disk.
% Disks of a whole bunch of sizes are generated with their
% centra at different locations. We record the relative errors
% made for all (dx,dy,r): p1, p2 and pInf errors.

% Cris Luengo

if 1

clear

% Determine what to measure
w = 8:0.02:25;                % Radii
c = 0:0.01:0.5;               % Centra
N = length(w);

% Measure errors
err1 = zeros(length(c),length(c));
err2 = zeros(length(c),length(c));
errInf = zeros(length(c),length(c));
sz = w(end)*2+5;
xxx = xx([sz,sz]);
yyy = yy([sz,sz]);
count = 0;
tcount = length(c).^2;
for ii=1:length(c)
   for jj=1:ii
      x = xxx+c(ii);
      y = yyy+c(jj);
      r = sqrt(x^2+y^2);
      err = zeros(N,1);
      for nn=1:N
         s = sum(r <= w(nn));
         err(nn) = (s / (pi*w(nn).^2)) - 1; % relative error
      end
      err1(ii,jj) = mean(abs(err));
      err2(ii,jj) = sqrt(sum(err.^2)/N);
      errInf(ii,jj) = max(abs(err));
      count = count+1;
      if (ii~=jj)
         err1(jj,ii) = err1(ii,jj);
         err2(jj,ii) = err2(ii,jj);
         errInf(jj,ii) = errInf(ii,jj);
         count = count+1;
      end
   end
   disp([num2str(count/tcount*100),'%'])
end

clear err x xxx y yyy ii jj nn s N r sz count tcount
save disk_origin

end

%%%

if 1

load disk_origin

err2 = dip_image(err2,'sfloat');
pe2 = findcoord(err2==min(err2));
pe2 = pe2(1,:);  % symmetric, 1 minimum


% Measure errors again
w = 1:0.02:20;              % Radii
cx = [0,0.25,c(pe2(1)+1)];  % Centra
cy = [0,0.25,c(pe2(2)+1)];
N = length(w);
err = zeros(length(cx),N);
sz = w(end)*2+5;
xxx = xx([sz,sz]);
yyy = yy([sz,sz]);
for ii=1:length(cx)
   x = xxx+cx(ii);
   y = yyy+cy(ii);
   r = sqrt(x^2+y^2);
   for nn=1:N
      s = sum(r <= w(nn));
      err(ii,nn) = (s / (pi*w(nn).^2)) - 1; % relative error
   end
end


% Minimal radius, 1% relative error:
%min0 = find(abs(err(1,:))>0.01); min0 = w(min0(end)+1)
%min4 = find(abs(err(2,:))>0.01); min4 = w(min4(end)+1)
%minmin = find(abs(err(3,:))>0.01); minmin = w(minmin(end)+1)

% Minimal radius, 5% relative error:
%min0 = find(abs(err(1,:))>0.05); min0 = w(min0(end)+1)
%min4 = find(abs(err(2,:))>0.05); min4 = w(min4(end)+1)
%minmin = find(abs(err(3,:))>0.05); minmin = w(minmin(end)+1)

% Plot
figure('position',[300,300,700,400])
hold on
plot(w,squeeze(double(err(1,:))),'linestyle','-','color',[0.5,0.5,0.5])
plot(w,squeeze(double(err(3,:))),'linestyle','-','color',[0,0,0])
set(gca,'xlim',w([1,end]),'ylim',[-0.2,0.2])
legend({'(0,0)','minimal P^2 error'},4)
ylabel('relative error in area')
xlabel('radius')
box on

set(gcf,'PaperPositionMode','auto')
print -depsc2 -r600 disk_origin_errorplot.eps

end

%%%

if 1

load disk_origin

figure('position',[500,500,550,300])
surf(c,c,err2,'facecolor',[0.75,0.75,0.75],'edgecolor','none')
hold on
stepsz = 5;
for ii=1:stepsz:length(c)
   line(repmat(c(ii),1,length(c)),c,err2(ii,:),'color',[0,0,0]);
end
for ii=1:stepsz:length(c)
   line(c,repmat(c(ii),1,length(c)),err2(:,ii),'color',[0,0,0]);
end
set(gca,'box','on','projection','perspective')
set(gca,'position',[0.25,0.2,0.5,0.75])
set(gca,'zlim',[0,0.01],'xlim',c([1,end]),'ylim',c([1,end]))
view(-73.5,62)
zlabel('Root Mean Square Error','rotation',103)
xlabel('x-coordinate of center','position',[0.20,-0.1,0],'verticalalignment','middle','horizontalalignment','center','rotation',67);
ylabel('y-coordinate of center','position',[-0.15,0.25,0],'verticalalignment','middle','horizontalalignment','center','rotation',-8)
colormap([128:255;128:255;128:255]'/255)
h = camlight('left');
lighting phong
material dull
set(h,'style','infinite','position',[-0.5,0.5,0.08])

set(gcf,'PaperPositionMode','auto')
print -deps2 -r450 disk_origin_mean2error.eps

end
