%GRANUL_RECT   Obtains a particle size distribution
%  distr = granul_rect(in,widths,polarity)
%  returns a granulometry using rectangular structuring elements.
%
% PARAMETERS:
%  in: input image
%  widths: sample locations; these are the widths of the rectangles used as SEs.
%  polarity: whether to use openings or closings. One of these options:
%     'dark': uses closings and measures dark particles agains a light background.
%     'light': uses openings and measures light particles against a dark background.

% Cris Luengo, July-August 2006

function distr = granul_rect(in,widths,polarity)

% Check input
if isa(in,'dip_image_array') | isempty(in) | ndims(in)~=2
   error('2D scalar input image expected.')
end
if ~isnumeric(widths) | ~isvector(widths)
   error('WIDTHS should be a numeric vector')
end
if strcmp(polarity,'dark')
   polarity = 0;
else
   polarity = 1;
end

% Compute distribution
levels = length(widths);
distr = zeros(1,levels);
maxim = max(in);
minim = min(in);
offset = mean(in);
if polarity==0 % this is a closing sieve..
   gain = 1/(maxim-offset);
else % this is an opening sieve...
   gain = 1/(minim-offset);
end
for level=1:levels
   if polarity==0
      tmp = dip_closing(in,[],[1,1]*widths(level),'rectangular');
   else
      tmp = dip_opening(in,[],[1,1]*widths(level),'rectangular');
   end
   tmp = mean(tmp);
   distr(level) = (tmp-offset)*gain;
end
