% TESTIM
% Computes the granulometry with sampled disks, centered at (0,0) and
% at the optimal location (0.19,0.31), of a sequence of 13 images that
% are identical except for a rotation. The granulometry is applied
% with high scale sampling density on each image after 4 different
% interpolation factors (1,2,4 and 8). The function TESTIM_GRAPH then
% will generate plots that show how interpolation and shifting the
% disk origin improve isotropy, and how the scale sampling density
% is limited by the discretization error.

% Cris Luengo
% 16 Nov 2000
% 17 Feb 2003
%  2 Aug 2006 (just compacting code & splitting out testim_graph.m)

clear all
w = 2.^((1:22)/4);            % kernel sizes
len = length(w);
zoom = [1,2,4,8];             % zoom factors - consider removing the 8 for large reduction in computation time.
Nz = length(zoom);
angles = [0.1:0.3:3.9]*pi/16; % image rotation angles
Na = length(angles);
bn = zeros(len,Na,Nz);         % output array - no center
bc = zeros(len,Na,Nz);         % output array - with center
window = 255*gaussianedgeclip(rr-120);
linesetH1x = (-128:20:128)+randn(1,13);
linesetV1y = (-128:20:128)+randn(1,13);
linesetH2x = (-128:20:128)+10+randn(1,13);
linesetV2y = (-128:20:128)+10+randn(1,13);
xim = xx;
yim = yy;
for ii=1:Na
   disp(['angle = ',num2str(angles(ii))])
   x = xim*cos(angles(ii))+yim*sin(angles(ii));
   y = xim*sin(angles(ii))-yim*cos(angles(ii));
   out1x = newim+255;
   out1y = newim+255;
   out2x = newim+255;
   out2y = newim+255;
   for kk=1:13
      out1x = min(out1x,abs(x-linesetH1x(kk)));
      out1y = min(out1y,abs(y-linesetV1y(kk)));
      out2x = min(out2x,abs(x-linesetH2x(kk)));
      out2y = min(out2y,abs(y-linesetV2y(kk)));
   end
   img1 = 255*gaussianlineclip(min(out1x,out1y),1.2,1);
   img2 = 128*gaussianlineclip(min(out2x,out2y),1.2,1);
   img = max(img1,img2);
   img = max(img,window);
   writeim(dip_image(img,'uint8'),['testim_rot_',num2str(ii,'%.2d')],'tiff')
   for jj=1:Nz
      disp(['   zoom = ',num2str(zoom(jj))])
      if zoom(jj)>1
         tmp = dip_resampling(img,[1,1].*zoom(jj),[0,0],'3-cubic');
      else
         tmp = img;
      end
      offset = mean(tmp);
      gain = 1/(max(tmp)-offset);
      sz = [1,1]*(w(end)*zoom(jj)+1);
      bn(:,ii,jj) = granulometry(tmp,w.*zoom(jj),1,40000,1,1,{},'dark')';
      bc(:,ii,jj) = granulometry(tmp,w.*zoom(jj),1,40000,1,1,{'usecenter'},'dark')';
   end
end

save testim bn bc w zoom angles
