% TESTIM_GRAPH
% Draws graphs with the data produced by TESTIM.

% Cris Luengo

function testim_graph

zoom = ''; % needed because we load this variable and hide a function call
load testim
sub_testim_graph(bn,w,zoom,'nocenter')
sub_testim_graph(bc,w,zoom,'center')


function sub_testim_graph(b,w,zoom,fname)

t = repmat(b(end,:,:),[size(b,1),1,1]);
b = b./t; % normalization for comparision purposes. The normalization is off because of the interpolation.
w = w(:);
Nz = length(zoom);

% 4 samples/octave
w4 = w;
I4 = log2(w);
b4 = b;
wx = repmat(w4,[1,size(b4,2),size(b4,3)]);
d4 = 8*(b4(2:end,:,:)-b4(1:end-1,:,:))./(wx(2:end,:,:)-wx(1:end-1,:,:));
% 2 samples/octave
b2 = b4(2:2:end,:,:);
I2 = I4(2:2:end);
w2 = w4(2:2:end);
wx = repmat(w2,[1,size(b2,2),size(b2,3)]);
d2 = 8*(b2(2:end,:,:)-b2(1:end-1,:,:))./(wx(2:end,:,:)-wx(1:end-1,:,:));
% 1 samples/octave
b1 = b2(2:2:end,:,:);
I1 = I2(2:2:end);
w1 = w2(2:2:end);
wx = repmat(w1,[1,size(b1,2),size(b1,3)]);
d1 = 8*(b1(2:end,:,:)-b1(1:end-1,:,:))./(wx(2:end,:,:)-wx(1:end-1,:,:));
% Statistics on the distributions
e4 = squeeze(std(d4,0,2));
d4 = squeeze(mean(d4,2));
b4 = squeeze(mean(b4,2));
e2 = squeeze(std(d2,0,2));
d2 = squeeze(mean(d2,2));
b2 = squeeze(mean(b2,2));
e1 = squeeze(std(d1,0,2));
d1 = squeeze(mean(d1,2));
b1 = squeeze(mean(b1,2));
% Plots
figure; set(gcf,'position',[490,130,775,800])
for ii=1:Nz
   subplot(Nz,3,(ii-1)*3+3)
   bar_ex(I4,d4(:,ii),[0.5,0.5,0.5],[0.75,0.75,0.75])
   hold on
   plot([0;I4],[0;b4(:,ii)],'k-','linewidth',2)
   h = errorbar('v6',I4(1:end-1)+I4(1)/2,d4(:,ii),e4(:,ii),'k-');
   delete(h(2))
   set(gca,'XLim',[0,I4(end-3)],'XTick',I1,'XTickLabel',w1,'YLim',[0,1])

   subplot(Nz,3,(ii-1)*3+2)
   bar_ex(I2,d2(:,ii),[0.5,0.5,0.5],[0.75,0.75,0.75])
   hold on
   plot([0;I2],[0;b2(:,ii)],'k-','linewidth',2)
   h = errorbar('v6',I2(1:end-1)+I2(1)/2,d2(:,ii),e2(:,ii),'k-');
   delete(h(2))
   set(gca,'XLim',[0,I4(end-3)],'XTick',I1,'XTickLabel',w1,'YLim',[0,1])

   subplot(Nz,3,(ii-1)*3+1)
   bar_ex(I1,d1(:,ii),[0.5,0.5,0.5],[0.75,0.75,0.75])
   hold on
   plot([0;I1],[0;b1(:,ii)],'k-','linewidth',2)
   h = errorbar('v6',I1(1:end-1)+I1(1)/2,d1(:,ii),e1(:,ii),'k-');
   delete(h(2))
   set(gca,'XLim',[0,I4(end-3)],'XTick',I1,'XTickLabel',w1,'YLim',[0,1])

   text(-2,0.75,[num2str(zoom(ii)),'x'],'FontSize',14)
end
subplot(Nz,3, 2); text(3,1.2,'Number of scale samples','HorizontalAlignment','center','FontSize',14)
subplot(Nz,3, 4); text(-3,-0.2,'Interpolation factor','HorizontalAlignment','center','FontSize',14,'Rotation',90) % this one works well if Nz==4
set(gcf,'PaperPositionMode','Auto')
print('-deps','-r600',['testim_distributions_',fname,'.eps'])
